import groovy.json.JsonSlurper
import ch.nevis.esauth.auth.engine.AuthResponse
import ch.nevis.esauth.util.httpclient.api.HttpClient

import io.opentelemetry.api.trace.Span

def getHeader(String name) {
    def inctx = request.getLoginContext()
    // case-insensitive lookup of HTTP headers
    def map = new TreeMap<>(String.CASE_INSENSITIVE_ORDER)
    map.putAll(inctx)
    return map['connection.HttpHeader.' + name]
}

def clearIdmSessionAttributes() {
  def s = request.getAuthSession(true)
  def sessionKeySet = new HashSet(session.keySet())
  sessionKeySet.each { key ->
    if ( key ==~ /ch.nevis.idm.*/ || key ==~ /ch.adnovum.nevisidm.*/ ) {
      s.removeAttribute(key)
    }
  }
}

def verification_request_template = '''
{ "presentation_definition": {
    "id": "{{UUID}}",
    "name": "AGOV Verification",
    "purpose": "AGOV Login",
    "format": {
      "vc+sd-jwt": {
        "sd-jwt_alg_values": [
          "ES256"
        ],
        "kb-jwt_alg_values": [
          "ES256"
        ]
      }
    },
    "input_descriptors": [
      {
        "id": "agov-all-attributes",
        "name": "AGOV Identity Verification",
        "purpose": "verification and authentication",
        "format": {
          "vc+sd-jwt": {
            "sd-jwt_alg_values": [
              "ES256"
            ],
            "kb-jwt_alg_values": [
              "ES256"
            ]
          }
        },
        "constraints": {
          "fields": [
            {
              "path": [
                "$.family_name"
              ]
            },
            {
              "path": [
                "$.given_name"
              ]
            },
            {
              "path": [
                "$.birth_date"
              ]
            },
            {
              "path": [
                "$.sex"
              ]
            },
            {
              "path": [
                "$.place_of_origin"
              ]
            },
            {
              "path": [
                "$.birth_place"
              ]
            },
            {
              "path": [
                "$.nationality"
              ]
            },
            {
              "path": [
                "$.personal_administrative_number"
              ]
            },
            {
              "path": [
                "$.document_number"
              ]
            },
            {
              "path": [
                "$.issuance_date"
              ]
            },
            {
              "path": [
                "$.expiry_date"
              ]
            },
            {
              "path": [
                "$.issuing_authority"
              ]
            },
            {
              "path": [
                "$.issuing_country"
              ]
            }
          ]
        }
      }
    ]
  }
}
'''


// ---------------
// check, whether we are still processing the correct AuthnRequest
if (inargs.containsKey('authRequestId') && (inargs['authRequestId'] != session['ch.nevis.auth.saml.request.id'])) {
    // wrong request, "force" a timeout
    LOG.debug('authentication timeout enforced, due to concurrent requests -> return a 408')

    response.setIsDirectResponse(true)
    response.setContentType('text/html; charset=UTF-8')
    response.setContent('Timeout')
    response.setHttpStatusCode(205)
    response.setHeader('IDP-AUTH', 'Timeout')

    // CONTINUE to keep the other request beeing processed
    response.setStatus(AuthResponse.AUTH_CONTINUE)
    return
}

if (inargs['oid4vp'] == 'ERROR') {
    response.setResult('error')
    return
}

if (inargs['oid4vp'] == 'SUCCEEDED') {
    response.setResult('ok')
    return
}


def sess = request.getAuthSession(true)

HttpClient httpClient = HttpClients.create(parameters)
def spanCtxt = Span.current().getSpanContext()
def traceparent = "00-${spanCtxt.getTraceId()}-${spanCtxt.getSpanId()}-${spanCtxt.getTraceFlags().asHex()}"

if (!session['agov.eid.verification']) {
    def endPoint = "${parameters.get('eidVerifierBaseUrl')}/api/v1/verifications"

	try {
		def httpResponse = Http.post()
			.url(endPoint)
			.header("Accept", "application/json")
			.header("traceparent", traceparent)
			.entity(Http.entity()
				.content(verification_request_template.replaceAll("\\{\\{UUID\\}\\}", UUID.randomUUID().toString()))
				.contentType("application/json")
				.build())
			.build()
			.send(httpClient)


		if (httpResponse.code() != 200) {
            LOG.debug("Result: ${httpResponse}")
			response.setResult('error')
			return
		}

		def json = new JsonSlurper().parseText(httpResponse.bodyAsString())
        LOG.debug("Result: ${json}")

		sess.setAttribute('agov.eid.verification', 'true')
		sess.setAttribute('agov.eid.verification.id', json.id)
		sess.setAttribute('agov.eid.verification.link', json.verification_url)

        if (json.state != 'PENDING') {
			response.setResult('error')
			return
        }
	} catch(Exception e) {
		LOG.error("Eid verification failed: $e")
		response.setResult('error')
		return
	}
}

if (getHeader('Content-Type') == 'application/json' && inargs.containsKey('o.id.v')) {
	try {

        def endPoint = "${parameters.get('eidVerifierBaseUrl')}/api/v1/verifications/${inargs['o.id.v']}"

		def httpResponse = Http.get()
			.url(endPoint)
			.header("Accept", "application/json")
			.header("traceparent", traceparent)
			.build()
			.send(httpClient)


		if (httpResponse.code() != 200) {
            LOG.debug("Result: ${httpResponse}")
			response.setResult('error')
			return
		}

		def json = new JsonSlurper().parseText(httpResponse.bodyAsString())

        if (json.state == 'SUCCESS') {
            sesss.setAttribute('ch.nevis.idm.User.firstName', json.wallet_response.given_name)
            sesss.setAttribute('ch.nevis.idm.User.lastName', json.wallet_response.family_name)
            sesss.setAttribute('ch.nevis.idm.User.birthDate', json.wallet_response.birth_date)
            sesss.setAttribute('ch.nevis.idm.User.gender', json.wallet_response.sex)
            sesss.setAttribute('ch.nevis.idm.User.prop.svnr', json.wallet_response.personal_administrative_number)
            sesss.setAttribute('ch.nevis.idm.User.prop.placeOfBirth', json.wallet_response.birth_place)
            sesss.setAttribute('ch.nevis.idm.User.prop.eIdNumber', json.wallet_response.personal_administrative_number)
            sesss.setAttribute('ch.nevis.idm.User.prop.nationality', json.wallet_response.nationality)
            sesss.setAttribute('ValidFrom', json.wallet_response.issuance_date)
            sesss.setAttribute('ValidTo', json.wallet_response.expiry_date)
            sesss.setAttribute('authenticatedWith', "EID")

        }


	} catch(Exception e) {
		LOG.error("Eid verification failed: $e")
		response.setResult('error')
		return
	}

}

// if we reach this place, display GUI
response.setStatus(AuthResponse.AUTH_CONTINUE)
return
